function fns = ConstructRFLLDMFLProb_Grass(D, N, lambda, fns)
% Low-dimensional multitask feature learning problem
% 
% D is the dataset, stroed in cell.
%   D{i}.X
%   D{i}.y
% N is number of sample
% fns is a structure related to Problem, and contains 
%    fns.cost, 
%    fns.egrad, 
%    fns.partialegrad, 

    fns.cost  = @(U) cost(U, D, N, lambda);
    fns.egrad = @(U) partialegrad(U, D, lambda, 1:N);
    fns.partialegrad = @(U, S) partialegrad(U, D, lambda, S);
    
    fns.grad = @(U) fns.M.egrad2rgrad(U, fns.egrad(U));
    fns.partialgrad = @(U, S) fns.M.egrad2rgrad(U, fns.partialegrad(U, S));
    
    fns.NMSE = @(U, D, N) NMSE(U, D, N, lambda);
    fns.prediction = @(U, D) prediction(U, D, N, lambda);

    function output = cost(U, D, N, lambda)
        tmp = 0;
        for i = 1:N
            tmp = tmp + singlecost(U, D{i}, lambda);
        end
        output = tmp / N;
    end
    
    % partial Euclidean gradient
    function output = partialegrad(U, D, lambda, S)
        tmp = zeros(size(U));
        for i = reshape(S, 1, length(S))
            tmp = tmp + singleegrad(U, D{i}, lambda);
        end
        output = tmp / length(S);
    end

    function [output, tmpm, tmpd, tmpy] = NMSE(U, D, N, lambda)
        tmpm = 0;
        tmpd = 0;
        tmpy = [];
        for i = 1:N
            tmpm = tmpm + singleMSE(U, D{i}, lambda);
            tmpd = tmpd + length(D{i}.y);
            tmpy = [tmpy; D{i}.y];
        end
        output = tmpm / tmpd / var(tmpy);        
    end

    function output = prediction(U, D, N, lambda)
        output = cell(N, 1);
        for i = 1:N
            output{i} = singlePrediction(U, D{i}, lambda);
        end
    end
    
%% single point 

    % computing single task cost
    function output = singlecost(U, Di, lambda)
        y = Di.y;
        XUwu = singlePrediction(U, Di, lambda);

        tmp = XUwu - y;
        output = 0.5 * sum(sum(tmp.^2));
    end
    
    % single task euclidean gradient for F_ij
    function output = singleegrad(U, Di, lambda)
        y = Di.y;
        X = Di.X;
        [XUwu, wu, HU, XU] = singlePrediction(U, Di, lambda);

        tmp = (XUwu - y);
        part1 = X' * tmp * wu';
        part2 = (X' * y) * (tmp' * XU) / HU;         
        part3 = (X' * XU / HU) * (XU' * tmp) * (y' * XU) / HU;
        part4 = (X' * XU / HU) * (XU' * y) * (tmp' * XU / HU);
        output = part1 + part2 - part3 - part4;
    end        

    function output = singleMSE(U, Di, lambda)
        y = Di.y;
        XUwu = singlePrediction(U, Di, lambda);

        tmp = XUwu - y;
        output = sum(sum(tmp.^2));
    end

    % single task prediction
    function [output, wu, HU, XU]= singlePrediction(U, Di, lambda)
        n = size(U,2);
        y = Di.y;
        X = Di.X;
        XU = X * U;
        HU = (XU' * XU - 2 * lambda * eye(n));  
        wu = HU \ U' * (X' * y);
        
        output = XU * wu;
    end
end





